#
# This script will perform a potential energy scan using the Gaussian Program
# It is preferred to using the internal SCAN function, or opt=modredundant, as failure of one
# scan point will not stop the script
#
# Version 28.09.2011
# Changed to include the scan direction functionality, which means no more dramatic jumps
# in the second (and third if present) variable(s)
#
# Version 13.03.2017
# Changed to be a 2D script only and modified with additional conditional statements to allow restarting optimisation
# jobs in case convergence is not achieved.

# The following applies for a PBS queuing system on pople, can be ignored if running on local computer

#!/bin/tcsh
#PBS -l nodes=1:ppn=12
#PBS -q long
#PBS -W x=PARTITION:DEFAULT
#PBS -l mem=24000MB
#PBS -M duncan.wild@uwa.edu.au
#PBS -m abe

#uncomment the following if running on a super computer, and update the PBS queue options above

#module load gaussian/g09

#
# Below is the start of the script itself
#


#!/bin/sh
#
# Give the scan job a title here
# The short title will form the name of the scan output file
#

title="Cl-CO anion at mp2/aug-cc-pvdz"
short_title="ClCOanionpvdz"

#
# The next section is where the scan variables are set
#
# _start is the starting value of the variable
# _del is the step size (in degs or Angstrom)
# -steps is the number of steps for each variable 
#
# The example in the script is for the relaxed 2D PES scan of Cl...CO anion at pvdz.
#
# X
# X  1 1.0
# C  2 R1 1 90.0
# O  2 R1 1 90.0 3 180.0 0
# Cl 2 var1 3 var2 1 180.0 0
# Variables:
# R1 = 0.564
#
# In the above, the intermolecular separation and angle made between line to the bond mid point and bond angle
# are the scanned variables, i.e. var1 and var2, while the CO bond length is the optimised variable.
#

var2_start=0.5
var2_del=10.0
var2_steps=9

#
# The next is the value of i and j for the loops below. This is needed if a restart is necessary
# Look to the $shortname.ijk output file for the last value of i and j and set these below
# If this is the first time the scan is run, then set all to 1
#
# If the run is to be extended, then keep the same scan "start" and "del" variables, change the "steps" variable, and set
# i_num and j_num to the end of the first scan run. Don't forget to change the short name variable!
#
# The dirnj variables are new and determine the direction of scan of the 2nd variable
#

j_num=1

#
# The next part will change the number of steps so it is in line with what
# is done with Gaussin 03 (i.e. do the first point, and then n steps more)
# Comment these lines out if you have already accounted for this above
#

var2_steps=`expr $var2_steps + 1`

# The following variables are used in the loop below as a test if the job produced an Energy value and had Normal termination	

Succ=0		
try=0

# The following variables deal with the keywords of the Gaussian job file

keymaster="# mp2 opt=(z-matrix,maxcycle=100) NoSymm" # Put your level of theory and opt keywords here, NoSymm is important if the scan breaks symmetry!
basis="aug-cc-pvdz"  # basis set goes here
keywords="$keymaster $basis"  # sets up the keywords for the start of the run

# If you are mid way through a scan, then the following will append the Guess=read keyword to take the guess from 
# the checkpoint file which remains from previous scan job

i_j=`expr $j_num`
if [ $i_j -gt 1 ]
    then
    	keywords="$keywords Guess=Read"
fi

#
# Print info to the .log and .ijk files
#

NumPoints=`echo "$var2_steps" | bc` # Calculates the total number of points in the scan
echo $title > $short_title.log
echo "This scan will produce $NumPoints values" >> $short_title.log
echo "VAR2	Energy	# of Tries" >> $short_title.log
echo "J" > $short_title.ijk
echo "VAR2,Energy" > $short_title.csv

# Make a directory for holding the log files

mkdir scan_log_files

#
# If restarting, then the direction of scan given by dirnj is important, below it is used to define the starting point of variable 2
#

var2=`echo " (($j_num-1) * $var2_del) + $var2_start" | bc`	# The scan direction of variable 1 does not need a direction change

# The following ensures that 0.0 is provided to the Gaussian 09 input, rather than 0

if [ $var2 -eq "0" ]
    then
    var2="0.0"
fi

#
# Next comes the loops which will scan through the variables above
# More can be put in, you only need to increase the number of nested loops
# and also replicate the scan direction variables for all but the first variable
#


        for (( j=$j_num; j<=$var2_steps; j++))
        do
	    while [ $Succ -lt 1 ]
	    do	
		    echo "%chk=temp.chk" > temp.inp
		    echo "%mem=13250MB" >> temp.inp		# Change the RAM according to the computer system
		    echo "%nproc=8" >> temp.inp			# Change the number of processors
 		    echo "$keywords" >> temp.inp
		    echo "" >> temp.inp
		    echo "Scan temp file" >> temp.inp
		    echo "" >> temp.inp
		    echo "-1,1" >> temp.inp
			if [ "$try" -eq "0" ]			# The 'if then' statement was introduced as Gaussian can't handle
				then				# restarting a job using geom=checkpoint and retaining the molecular geometry
				echo "X									
X 1 1.0
C 2 R1 1 90.0
O 2 R1 1 90.0 3 180.0 0
Cl 2 3.0 3 $var2 1 180.0 0
Variables:
R1 = 0.564
" >> temp.inp
				echo ""  >> temp.inp
			else
				"" >> temp.inp
			fi	
#
# The next section will start the program, wait for completion and
# search for the energy value. If Normal termination is not found in the log file, then the optimisation job is started again
# using Geom=Checkpoint Guess=Read and ChkBasis.
#
# The try variable keeps track of how many times the restart occurs
# 
		    g09 < temp.inp > temp.log 2>&1
			wait
			NT=`grep "Normal termination" temp.log | awk '{print $1}'`
			En=`grep EUMP2 temp.log | tail -1 | awk '{print $6}'`			# search for the last instance of the energy, and grab it.
			if [ -z $NT ] 
			  then
			  if [ "$try$" -lt "3"]							# only try three times to converge the geometry
			     then
				Succ=0
				keywords="$keymaster Geom=Checkpoint ChkBasis Guess=Read"
				try=`expr $try + 1`
				cp temp.log Point.$var2.log_Try$try			# added 13.03.17 to keep the input file and 
				cp temp.inp Point.$var2.inp_Try$try			# output from failed runs to inspect the error
			     else
			    	Succ=1
				En="Fail"
			  fi
			else
			     Succ=1
			fi  
		done
	    echo "$var2	$En	$try" >> $short_title.log
		echo "$var2,$En" >> $short_title.csv				# save the scan data as a csv file for easy import into graphing programs
	    echo "$j" >> $short_title.ijk
	    mv temp.log scan_log_files/Point_"$var1"_"$var2".log	# Save the temporary log file for viewing later
	    rm temp.inp
	    keywords="$keymaster $basis Guess=Read"			# At the end of a successful run, change the keywords to include Guess=Read
	    try=0
	    Succ=0
	    var2=`echo "$var2 + $var2_del" | bc`
		if [ $var2 -eq "0" ]									# ensures that 0 is expressed as 0.0 in the Gaussian input
			then
			var2="0.0"
		fi
	    j_num=1
        done

echo "Scan finished!" >> $short_title.log 
echo "Scan finished!" >> $short_title.ijk
sed -i 's/D+/E+/g' $short_title.*

#
# Clean up temporary files (add below here if needs be)
#


